package com.example.mobile_labs.api

import com.example.mobile_labs.api.models.EventRemote
import com.example.mobile_labs.api.models.PerformanceRemote
import com.example.mobile_labs.api.models.PersonRemote
import com.jakewharton.retrofit2.converter.kotlinx.serialization.asConverterFactory
import kotlinx.serialization.json.Json
import okhttp3.MediaType.Companion.toMediaType
import okhttp3.OkHttpClient
import okhttp3.logging.HttpLoggingInterceptor
import retrofit2.Retrofit
import retrofit2.http.Body
import retrofit2.http.DELETE
import retrofit2.http.GET
import retrofit2.http.POST
import retrofit2.http.PUT
import retrofit2.http.Path
import retrofit2.http.Query
import java.util.concurrent.TimeUnit

interface MyServerService {
    @GET("api/persons")
    suspend fun getPeople(
        @Query("_page") page: Int,
        @Query("_limit") limit: Int,
    ): List<PersonRemote>

    @GET("api/performances")
    suspend fun getPerformances(
        @Query("_page") page: Int,
        @Query("_limit") limit: Int,
    ): List<PerformanceRemote>

    @GET("api/events")
    suspend fun getEvents(
        @Query("_page") page: Int,
        @Query("_limit") limit: Int,
    ): List<EventRemote>

    @GET("api/performances/{id}")
    suspend fun getPerformance(
        @Path("id") id: Int,
    ): PerformanceRemote

    @GET("api/persons/{id}")
    suspend fun getPerson(
        @Path("id") id: Int,
    ): PersonRemote

    @GET("api/events/{id}")
    suspend fun getEvent(
        @Path("id") id: Int,
    ): EventRemote

    companion object {
        private const val BASE_URL = "http://10.0.2.2:8000/"

        @Volatile
        private var INSTANCE: MyServerService? = null

        private var _token: String = ""

        fun getInstance(): MyServerService {
            return INSTANCE ?: synchronized(this) {
                val logger = HttpLoggingInterceptor()
                logger.level = HttpLoggingInterceptor.Level.BASIC
                val client = OkHttpClient.Builder()
                    .connectTimeout(1, TimeUnit.DAYS)
                    .readTimeout(1, TimeUnit.DAYS)
                    .writeTimeout(1, TimeUnit.DAYS)
                    .retryOnConnectionFailure(false)
                    .callTimeout(1, TimeUnit.DAYS)
                    .addInterceptor(logger)
                    .addInterceptor {
                        val originalRequest = it.request()
                        if (_token.isEmpty()) {
                            it.proceed(originalRequest)
                        } else {
                            it.proceed(
                                originalRequest
                                    .newBuilder()
                                    .header("Authorization", "Bearer $_token")
                                    .method(originalRequest.method, originalRequest.body)
                                    .build()
                            )
                        }
                    }
                    .build()
                return Retrofit.Builder()
                    .baseUrl(BASE_URL)
                    .client(client)
                    .addConverterFactory(Json.asConverterFactory("application/json".toMediaType()))
                    .build()
                    .create(MyServerService::class.java)
                    .also { INSTANCE = it }
            }
        }

        fun setToken(token: String) {
            _token = token
        }
    }
}