package com.example.mobile_labs.api.events

import androidx.paging.ExperimentalPagingApi
import androidx.paging.Pager
import androidx.paging.PagingConfig
import androidx.paging.PagingData
import com.example.mobile_labs.api.MyServerService
import com.example.mobile_labs.api.models.toPerformanceWithPeople
import com.example.mobile_labs.api.performance.RestPerformanceRepository
import com.example.mobile_labs.common.AppDataContainer
import com.example.mobile_labs.common.EventRepository
import com.example.mobile_labs.common.PerformanceRepository
import com.example.mobile_labs.database.AppDatabase
import com.example.mobile_labs.database.event.model.Event
import com.example.mobile_labs.database.event.model.EventWithPerformance
import com.example.mobile_labs.database.event.repository.OfflineEventRepository
import com.example.mobile_labs.database.performance.model.Performance
import com.example.mobile_labs.database.performance.model.PerformanceWithPeople
import com.example.mobile_labs.database.remotekeys.repository.OfflineRemoteKeyRepository
import kotlinx.coroutines.flow.Flow

class RestEventRepository(
    private val service: MyServerService,
    private val dbEventRepository: OfflineEventRepository,
    private val dbRemoteKeyRepository: OfflineRemoteKeyRepository,
    private val performanceRestRepository: RestPerformanceRepository,
    private val database: AppDatabase
) : EventRepository {
    override fun getAllWithPerformance(): Flow<PagingData<EventWithPerformance>> {

        val pagingSourceFactory = { dbEventRepository.getAllEventsPagingSource() }

        @OptIn(ExperimentalPagingApi::class)
        return Pager(
            config = PagingConfig(
                pageSize = AppDataContainer.LIMIT,
                enablePlaceholders = false
            ),
            remoteMediator = EventRemoteMediator(
                service,
                dbEventRepository,
                dbRemoteKeyRepository,
                performanceRestRepository,
                database,
            ),
            pagingSourceFactory = pagingSourceFactory
        ).flow
    }
}