package com.example.mobile_labs.api.models

import com.example.mobile_labs.api.MyServerService
import com.example.mobile_labs.database.performance.model.Performance
import com.example.mobile_labs.database.performance.model.PerformanceWithPeople
import com.example.mobile_labs.database.person.model.Person
import kotlinx.serialization.Serializable

@Serializable
data class PerformanceRemote(
    val id: Int = 0,
    val title: String = "",
    val description: String = "",
    val author_id: Int = 0,
    val director_id: Int = 0,
    val image_url: String = "",
    val preview_image_url: String = "",
    val actors: List<Int> = listOf()
)

fun PerformanceRemote.toPerformance(): Performance = Performance(
    id,
    title,
    description,
    author_id,
    director_id,
    image_url,
    preview_image_url
)

suspend fun PerformanceRemote.toPerformanceWithPeople(service: MyServerService): PerformanceWithPeople {
    val actorsList = ArrayList<Person>();
    actors.forEach {id ->
        actorsList.add(service.getPerson(id).toPerson())
    }

    return PerformanceWithPeople(
        service.getPerformance(id).toPerformance(),
        service.getPerson(author_id).toPerson(),
        service.getPerson(director_id).toPerson(),
        actorsList.toList()
        )
}