package com.example.mobile_labs.api.people

import android.util.Log
import androidx.paging.ExperimentalPagingApi
import androidx.paging.Pager
import androidx.paging.PagingConfig
import androidx.paging.PagingData
import com.example.mobile_labs.api.MyServerService
import com.example.mobile_labs.common.AppDataContainer
import com.example.mobile_labs.common.PersonRepository
import com.example.mobile_labs.database.AppDatabase
import com.example.mobile_labs.database.person.model.Person
import com.example.mobile_labs.database.person.repository.OfflinePersonRepository
import com.example.mobile_labs.database.remotekeys.repository.OfflineRemoteKeyRepository
import kotlinx.coroutines.flow.Flow

class RestPersonRepository(
    private val service: MyServerService,
    private val dbPersonRepository: OfflinePersonRepository,
    private val dbRemoteKeyRepository: OfflineRemoteKeyRepository,
    private val database: AppDatabase
) : PersonRepository {
    override fun getAllPeople(): Flow<PagingData<Person>> {

        val pagingSourceFactory = { dbPersonRepository.getAllPeoplePagingSource() }

        @OptIn(ExperimentalPagingApi::class)
        return Pager(
            config = PagingConfig(
                pageSize = AppDataContainer.LIMIT,
                enablePlaceholders = false
            ),
            remoteMediator = PeopleRemoteMediator(
                service,
                dbPersonRepository,
                dbRemoteKeyRepository,
                database,
            ),
            pagingSourceFactory = pagingSourceFactory
        ).flow
    }
}