package com.example.mobile_labs.api.performance

import androidx.paging.ExperimentalPagingApi
import androidx.paging.Pager
import androidx.paging.PagingConfig
import androidx.paging.PagingData
import com.example.mobile_labs.api.MyServerService
import com.example.mobile_labs.api.models.toPerformance
import com.example.mobile_labs.api.models.toPerformanceWithPeople
import com.example.mobile_labs.api.people.RestPersonRepository
import com.example.mobile_labs.common.AppDataContainer
import com.example.mobile_labs.common.PerformanceRepository
import com.example.mobile_labs.common.PersonRepository
import com.example.mobile_labs.database.AppDatabase
import com.example.mobile_labs.database.performance.model.Performance
import com.example.mobile_labs.database.performance.model.PerformanceWithPeople
import com.example.mobile_labs.database.performance.repository.OfflinePerformanceRepository
import com.example.mobile_labs.database.person.model.Person
import com.example.mobile_labs.database.person.repository.OfflinePersonRepository
import com.example.mobile_labs.database.remotekeys.repository.OfflineRemoteKeyRepository
import kotlinx.coroutines.flow.Flow

class RestPerformanceRepository(
    private val service: MyServerService,
    private val dbPerformanceRepository: OfflinePerformanceRepository,
    private val dbRemoteKeyRepository: OfflineRemoteKeyRepository,
    private val personRestRepository: RestPersonRepository,
    private val database: AppDatabase
) : PerformanceRepository {
    override fun getAllPerformances(): Flow<PagingData<Performance>> {

        val pagingSourceFactory = { dbPerformanceRepository.getAllPerformancesPagingSource() }

        @OptIn(ExperimentalPagingApi::class)
        return Pager(
            config = PagingConfig(
                pageSize = AppDataContainer.LIMIT,
                enablePlaceholders = false
            ),
            remoteMediator = PerformanceRemoteMediator(
                service,
                dbPerformanceRepository,
                dbRemoteKeyRepository,
                personRestRepository,
                database,
            ),
            pagingSourceFactory = pagingSourceFactory
        ).flow
    }

    override suspend fun getPerformance(uid: Int): PerformanceWithPeople =
        service.getPerformance(uid).toPerformanceWithPeople(service)
}