package com.example.mobile_labs.common

import android.content.Context
import com.example.mobile_labs.api.MyServerService
import com.example.mobile_labs.api.events.RestEventRepository
import com.example.mobile_labs.api.people.RestPersonRepository
import com.example.mobile_labs.api.performance.RestPerformanceRepository
import com.example.mobile_labs.database.event.repository.OfflineEventRepository
import com.example.mobile_labs.database.performance.repository.OfflinePerformanceRepository
import com.example.mobile_labs.database.person.repository.OfflinePersonRepository
import com.example.mobile_labs.database.AppDatabase
import com.example.mobile_labs.database.remotekeys.repository.OfflineRemoteKeyRepository

interface AppContainer {
    val personRestRepository: RestPersonRepository
    val eventRestRepository: RestEventRepository
    val performanceRestRepository: RestPerformanceRepository
}

class AppDataContainer(val context: Context) : AppContainer {
    val eventRepository: OfflineEventRepository by lazy {
        OfflineEventRepository(AppDatabase.getInstance(context).eventDao())
    }

    val performanceRepository: OfflinePerformanceRepository by lazy {
        OfflinePerformanceRepository(AppDatabase.getInstance(context).performanceDao())
    }

    val personRepository: OfflinePersonRepository by lazy {
        OfflinePersonRepository(AppDatabase.getInstance(context).personDao())
    }
    val remoteKeyRepository: OfflineRemoteKeyRepository by lazy {
        OfflineRemoteKeyRepository(AppDatabase.getInstance(context).remoteKeysDao())
    }

    override val personRestRepository: RestPersonRepository by lazy {
        RestPersonRepository(
            MyServerService.getInstance(),
            personRepository,
            remoteKeyRepository,
            AppDatabase.getInstance(context)
        )
    }

    override val performanceRestRepository: RestPerformanceRepository by lazy {
        RestPerformanceRepository(
            MyServerService.getInstance(),
            performanceRepository,
            remoteKeyRepository,
            personRestRepository,
            AppDatabase.getInstance(context)
        )
    }

    override val eventRestRepository: RestEventRepository by lazy {
        RestEventRepository(
            MyServerService.getInstance(),
            eventRepository,
            remoteKeyRepository,
            performanceRestRepository,
            AppDatabase.getInstance(context)
        )
    }

    companion object {
        const val TIMEOUT = 5000L
        const val LIMIT = 3
    }
}