package com.example.mobile_labs.database

import android.content.Context
import androidx.room.Database
import androidx.room.Room
import androidx.room.RoomDatabase
import androidx.sqlite.db.SupportSQLiteDatabase
import com.example.mobile_labs.database.event.dao.EventDao
import com.example.mobile_labs.database.event.model.Event
import com.example.mobile_labs.database.performance.dao.PerformanceDao
import com.example.mobile_labs.database.performance.dao.PerformancePersonDao
import com.example.mobile_labs.database.performance.model.Performance
import com.example.mobile_labs.database.performance.model.PerformancePersonCrossRef
import com.example.mobile_labs.database.person.dao.PersonDao
import com.example.mobile_labs.database.person.model.Person
import com.example.mobile_labs.database.remotekeys.dao.RemoteKeysDao
import com.example.mobile_labs.database.remotekeys.model.RemoteKeys
import com.example.mobile_labs.user.dao.UserDao
import com.example.mobile_labs.user.model.User
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import java.time.LocalDate

@Database(entities = [Performance::class, Person::class, Event::class, PerformancePersonCrossRef::class, User::class, RemoteKeys::class], version = 1, exportSchema = false)
abstract class AppDatabase : RoomDatabase() {
    abstract fun personDao(): PersonDao
    abstract fun performanceDao(): PerformanceDao
    abstract fun eventDao(): EventDao
    abstract fun performancePersonDao(): PerformancePersonDao
    abstract fun userDao(): UserDao
    abstract fun remoteKeysDao(): RemoteKeysDao

    companion object {
        private const val DB_NAME: String = "theatre-db"

        @Volatile
        private var INSTANCE: AppDatabase? = null

        private suspend fun populateDatabase() {
            INSTANCE?.let { database ->
                //people
                val personDao = database.personDao()
                val person1 = Person(1, "Иванов", "Иван", "https://bolshoi.ru/media/members/photos/86_ru_twyyatusbnifnzi_300x300_p.jpg")
                val person2 = Person(2, "Ксения", "Дудникова", "https://bolshoi.ru/media/members/photos/1756_ru_wkznwzklgosnbiq_300x300_p.jpg")
                val person3 = Person(3, "Евгения", "Сегенюк", "https://bolshoi.ru/media/members/photos/2908_ru_rycynrihdpzvgdj_300x300_p.jpg")
                val person4 = Person(4, "Петров", "Петр", "https://bolshoi.ru/media/members/photos/15820_ru_zvewuwadyjuywkh_300x300_p.jpg")

                personDao.insert(person1)
                personDao.insert(person2)
                personDao.insert(person3)
                personDao.insert(person4)

                val performanceDao = database.performanceDao()
                val performance1 = Performance(1, "Представление 1", "Описание представления 1", 1, 2, "https://img.freepik.com/free-photo/empty-stage-with-few-props-red-seats_181624-57595.jpg?w=1380&t=st=1696959739~exp=1696960339~hmac=2107448a1d874f1315b5cf246c71df70dc653707c7c8be4a38581d3b73f842ba", "https://www.theatreinparis.com/uploads/images/article/theatre-de-l-athenee-dr.jpg")
                val performance2 = Performance(2, "Представление 2", "Описание представления 2", 2, 3, "https://img.freepik.com/free-photo/empty-stage-with-few-props-red-seats_181624-57595.jpg?w=1380&t=st=1696959739~exp=1696960339~hmac=2107448a1d874f1315b5cf246c71df70dc653707c7c8be4a38581d3b73f842ba", "https://www.theatreinparis.com/uploads/images/article/theatre-de-l-athenee-dr.jpg")
                val performance3 = Performance(3, "Представление 3", "Описание представления 1", 3, 4, "https://img.freepik.com/free-photo/empty-stage-with-few-props-red-seats_181624-57595.jpg?w=1380&t=st=1696959739~exp=1696960339~hmac=2107448a1d874f1315b5cf246c71df70dc653707c7c8be4a38581d3b73f842ba", "https://www.theatreinparis.com/uploads/images/article/theatre-de-l-athenee-dr.jpg")

                performanceDao.insert(performance1)
                performanceDao.insert(performance2)
                performanceDao.insert(performance3)

                val performancePersonDao = database.performancePersonDao()
                val performancePerson1 = PerformancePersonCrossRef(1, 1)
                val performancePerson2 = PerformancePersonCrossRef(1, 2)
                val performancePerson3 = PerformancePersonCrossRef(1, 3)
                val performancePerson4 = PerformancePersonCrossRef(1, 4)

                performancePersonDao.insert(performancePerson1)
                performancePersonDao.insert(performancePerson2)
                performancePersonDao.insert(performancePerson3)
                performancePersonDao.insert(performancePerson4)

                val eventDao = database.eventDao()
                val event1 = Event(1, LocalDate.parse("2023-10-23"), 1)
                val event2 = Event(2, LocalDate.parse("2023-10-11"), 2)
                val event3 = Event(3, LocalDate.parse("2023-10-15"), 3)
                val event4 = Event(4, LocalDate.parse("2023-10-27"), 1)
                val event5 = Event(5, LocalDate.parse("2023-10-03"), 2)
                val event6 = Event(6, LocalDate.parse("2023-10-09"), 3)
                val event7 = Event(7, LocalDate.parse("2023-11-03"), 2)
                val event8 = Event(8, LocalDate.parse("2023-11-09"), 3)

                eventDao.insert(event1)
                eventDao.insert(event2)
                eventDao.insert(event3)
                eventDao.insert(event4)
                eventDao.insert(event5)
                eventDao.insert(event6)
                eventDao.insert(event7)
                eventDao.insert(event8)

                val userDao = database.userDao()
                val user1 = User(1, "test@example.com", "1234", "John", "Doe")
                val user2 = User(2, "test@example.com", "1234", "John", "Doe")
                userDao.insert(user1)
                userDao.insert(user2)
            }
        }

        fun getInstance(appContext: Context): AppDatabase {
            return INSTANCE ?: synchronized(this) {
                Room.databaseBuilder(
                    appContext,
                    AppDatabase::class.java,
                    DB_NAME
                )
//                    .addCallback(object : RoomDatabase.Callback() {
//                        override fun onCreate(db: SupportSQLiteDatabase) {
//                            super.onCreate(db)
//                            CoroutineScope(Dispatchers.IO).launch {
//                                populateDatabase()
//                            }
//                        }
//                    })
                    .build()
                    .also { INSTANCE = it }
            }
        }
    }
}