package com.example.mobile_labs.database.event.dao

import androidx.paging.PagingSource
import androidx.room.Dao
import androidx.room.Delete
import androidx.room.Insert
import androidx.room.Query
import androidx.room.Update
import com.example.mobile_labs.database.event.model.Event
import com.example.mobile_labs.database.event.model.EventWithPerformance
import com.example.mobile_labs.database.performance.model.Performance
import kotlinx.coroutines.flow.Flow
import java.time.LocalDate
import java.time.format.DateTimeFormatter

@Dao
interface EventDao {
    @Query("select * from events order by date asc")
    fun getAll(): Flow<List<Event>>

    @Query("select * from events where events.date > :dateFrom order by date collate nocase asc")
    fun getAllWithPerformance(dateFrom: String? = LocalDate.now().format(
        DateTimeFormatter.ofPattern("dd.MM"))): PagingSource<Int, EventWithPerformance>

    @Query("select * from events where events.uid = :uid")
    fun getByUid(uid: Int): Flow<EventWithPerformance>

    @Insert
    suspend fun insert(vararg event: Event)

    @Update
    suspend fun update(event: Event)

    @Delete
    suspend fun delete(event: Event)

    @Query("DELETE FROM events")
    suspend fun deleteAll()
}