package com.example.mobile_labs.database.event.repository

import androidx.paging.Pager
import androidx.paging.PagingConfig
import androidx.paging.PagingData
import androidx.paging.PagingSource
import com.example.mobile_labs.common.AppDataContainer
import com.example.mobile_labs.common.EventRepository
import com.example.mobile_labs.database.event.dao.EventDao
import com.example.mobile_labs.database.event.model.Event
import com.example.mobile_labs.database.event.model.EventWithPerformance
import com.example.mobile_labs.database.performance.model.Performance
import kotlinx.coroutines.flow.Flow

class OfflineEventRepository(private val eventDao: EventDao) : EventRepository {
    fun getAll(): Flow<List<Event>> = eventDao.getAll();
    override fun getAllWithPerformance(): Flow<PagingData<EventWithPerformance>> = Pager(
        config = PagingConfig(
            pageSize = AppDataContainer.LIMIT,
            enablePlaceholders = false,
        ),
        pagingSourceFactory = eventDao::getAllWithPerformance
    ).flow

    fun getEvent(uid: Int): Flow<EventWithPerformance?> = eventDao.getByUid(uid);

    suspend fun insertEvent(event: Event) = eventDao.insert(event);

    suspend fun updateEvent(event: Event) = eventDao.update(event);

    suspend fun deleteEvent(event: Event) = eventDao.delete(event);

    suspend fun clearEvents() = eventDao.deleteAll()

    suspend fun insertEvents(events: List<Event>) =
        eventDao.insert(*events.toTypedArray())

    fun getAllEventsPagingSource(): PagingSource<Int, EventWithPerformance> = eventDao.getAllWithPerformance()
}