package com.example.mobile_labs.database.performance.repository

import androidx.paging.Pager
import androidx.paging.PagingConfig
import androidx.paging.PagingData
import androidx.paging.PagingSource
import com.example.mobile_labs.common.PerformanceRepository
import com.example.mobile_labs.common.AppDataContainer
import com.example.mobile_labs.database.performance.dao.PerformanceDao
import com.example.mobile_labs.database.performance.model.Performance
import com.example.mobile_labs.database.performance.model.PerformanceWithPeople
import com.example.mobile_labs.database.person.model.Person
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.flow.first

class OfflinePerformanceRepository(private val performanceDao: PerformanceDao) :
    PerformanceRepository {
    override fun getAllPerformances(): Flow<PagingData<Performance>> = Pager(
        config = PagingConfig(
            pageSize = AppDataContainer.LIMIT,
            enablePlaceholders = false,
        ),
        pagingSourceFactory = performanceDao::getAll
    ).flow
    override suspend fun getPerformance(uid: Int): PerformanceWithPeople = performanceDao.getByUid(uid).first();

    suspend fun insertPerformance(performance: Performance) = performanceDao.insert(performance);

    suspend fun updatePerformance(performance: Performance) = performanceDao.update(performance);

    suspend fun deletePerformance(performance: Performance) = performanceDao.delete(performance);

    suspend fun clearPerformances() = performanceDao.deleteAll()

    suspend fun insertPerformances(performances: List<Performance>) =
        performanceDao.insert(*performances.toTypedArray())

    fun getAllPerformancesPagingSource(): PagingSource<Int, Performance> = performanceDao.getAll()
}