package com.example.mobile_labs.database.person.repository

import androidx.paging.Pager
import androidx.paging.PagingConfig
import androidx.paging.PagingData
import androidx.paging.PagingSource
import com.example.mobile_labs.common.PersonRepository
import com.example.mobile_labs.common.AppDataContainer
import com.example.mobile_labs.database.person.dao.PersonDao
import com.example.mobile_labs.database.person.model.Person
import kotlinx.coroutines.flow.Flow

class OfflinePersonRepository(private val personDao: PersonDao) : PersonRepository {
    override fun getAllPeople(): Flow<PagingData<Person>> = Pager(
        config = PagingConfig(
            pageSize = AppDataContainer.LIMIT,
            enablePlaceholders = false,
        ),
        pagingSourceFactory = personDao::getAll
    ).flow;
    fun getPerson(uid: Int): Flow<Person?> = personDao.getByUid(uid);

    suspend fun insertPerson(person: Person) = personDao.insert(person);

    suspend fun updatePerson(person: Person) = personDao.update(person);

    suspend fun deletePerson(person: Person) = personDao.delete(person);

    suspend fun clearPeople() = personDao.deleteAll()

    suspend fun insertPeople(people: List<Person>) =
        personDao.insert(*people.toTypedArray())

    fun getAllPeoplePagingSource(): PagingSource<Int, Person> = personDao.getAll()
}