package com.example.mobile_labs.ui.performance.view

import android.content.res.Configuration
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.material3.ExperimentalMaterial3Api
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.OutlinedTextField
import androidx.compose.material3.Surface
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.text.style.TextAlign
import androidx.compose.ui.tooling.preview.Preview
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import androidx.lifecycle.viewmodel.compose.viewModel
import coil.compose.AsyncImage
import com.example.mobile_labs.R
import com.example.mobile_labs.common.AppViewModelProvider
import com.example.mobile_labs.ui.theme.Mobile_LabsTheme

@OptIn(ExperimentalMaterial3Api::class)
@Composable
fun PerformanceView(
    viewModel: PerformanceViewModel = viewModel(factory = AppViewModelProvider.Factory)
) {
    val performanceWithPeople = viewModel.performanceUiState.performanceDetails;
    Column(
        horizontalAlignment = Alignment.CenterHorizontally,
        modifier = Modifier
            .padding(all = 10.dp)) {
        Text(text = performanceWithPeople.authorName, textAlign = TextAlign.Center)
        Text(text = performanceWithPeople.title, textAlign = TextAlign.Center, fontSize = 30.sp)
        AsyncImage(model = performanceWithPeople.imageURL, contentDescription = "performance image")
        OutlinedTextField(modifier = Modifier.fillMaxWidth(), value = performanceWithPeople.description, onValueChange = {}, readOnly = true,
            label = {
                Text(stringResource(id = R.string.performance_description))
            }
        )
        OutlinedTextField(modifier = Modifier.fillMaxWidth(), value = performanceWithPeople.actorsList, onValueChange = {}, readOnly = true,
            label = {
                Text(stringResource(id = R.string.performance_actors))
            }
        )
    }
}

@Preview(name = "Light Mode", showBackground = true, uiMode = Configuration.UI_MODE_NIGHT_NO)
@Preview(name = "Dark Mode", showBackground = true, uiMode = Configuration.UI_MODE_NIGHT_YES)
@Composable
fun PerformanceViewPreview() {
    Mobile_LabsTheme {
        Surface(
            color = MaterialTheme.colorScheme.background
        ) {
            PerformanceView()
        }
    }
}