package com.example.mobile_labs.ui.performance.view

import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.setValue
import androidx.lifecycle.SavedStateHandle
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import com.example.mobile_labs.database.performance.model.PerformanceWithPeople
import com.example.mobile_labs.common.PerformanceRepository
import kotlinx.coroutines.flow.filterNotNull
import kotlinx.coroutines.flow.first
import kotlinx.coroutines.launch

class PerformanceViewModel(
    savedStateHandle: SavedStateHandle,
    private val performanceRepository: PerformanceRepository
) : ViewModel() {

    var performanceUiState by mutableStateOf(PerformanceUiState())
        private set

    private val studentUid: Int = checkNotNull(savedStateHandle["id"])

    init {
        viewModelScope.launch {
            if (studentUid > 0) {
                performanceUiState = performanceRepository.getPerformance(studentUid)
                    .toUiState()
            }
        }
    }
}

data class PerformanceUiState(
    val performanceDetails: PerformanceDetails = PerformanceDetails(),
)

data class PerformanceDetails(
    val title: String = "",
    val authorName: String = "",
    val actorsList: String = "",
    val imageURL: String = "",
    val description: String = ""
)

fun PerformanceWithPeople.toDetails(): PerformanceDetails = PerformanceDetails(
    title = performance.title,
    authorName = String.format("%s %s", author.last_name, author.first_name),
    actorsList = buildString { for (actor in actors) append(actor.last_name + " " + actor.first_name + "\n") },
    imageURL = performance.imageURL,
    description = performance.description,
)

fun PerformanceWithPeople.toUiState(): PerformanceUiState = PerformanceUiState(
    performanceDetails = this.toDetails(),
)